(* balls in a rectangle *)

open Initialisation;;

let t = 0.3;;

let integr init dx =
  let rec x = dx *. t +. init fby x in
  x;;

let deriv x =
  let rec dx = ((0.0 fby x) -. x) /. t in
  dx;;

let front_haut cl = not (false fby cl) & cl;;
let bascule x =
  let rec bascule = if x then true fby not bascule
                else false fby bascule in
  bascule;;

let first x =
  let rec w = x fby w in
  w;;


(* clicks on buttons *)
let click_on_quit mouse_status =
  (front_haut (click mouse_status)) & (on_quit mouse_status);;

let click_on_restart mouse_status =
  (front_haut (click mouse_status)) & (on_restart mouse_status);;

let click_on_universe mouse_status =
  (front_haut (click mouse_status)) & (on_universe mouse_status);;

let click_on_create_circle mouse_status =
  (front_haut (click mouse_status)) & (on_create_circle mouse_status);;

let click_on_create_ball mouse_status =
  (front_haut (click mouse_status)) & (on_create_ball mouse_status);;

let click_on_create_sinusoide mouse_status =
  (front_haut (click mouse_status)) & (on_create_sinusoide mouse_status);;





(* the equation of a ball in a domain *)
(* with elastics chocs *)
let ball init_ball =
  
  (* initial position and initial speed *)
  let x0,y0,dx0,dy0 = init_ball in
  
  (* reactions on the domain *)
  let rec dx,dy = border (x0 fby x) (y0 fby y) (dx0 fby dx) (dy0 fby dy)

  and x = integr x0 dx
  
  and y = integr y0 dy in
  
  x,y;;

let main_ball init_ball =
  
  let x,y = ball init_ball in
  
  let x = truncate x in

  let y = truncate y in

  (* drawing the ball *)
  clear_plot (x fby x) (y fby y);
  plot x y;;


(* drawing a circle *)
let circle center radius =
  let dteta = 0.01 in
  let x0,y0 = first center in
  let radius = first radius in
  let rec teta = 0.0 -> pre teta +. dteta in
  x0 +. radius *. cos teta,
  y0 +. radius *. sin teta;;


let main_circle center radius =
  let x,y = circle center radius in
  plot (truncate x) (truncate y);;

(* a sinusoide *)
let sinusoide center radius speed =
  let x0,y0 = first center in
  let radius = first radius in
  let dx0,dy0 = first(speed) in
  let rec x = x0 -> pre x +. dx
  and yy = y0 -> pre yy +. dy
  and y = y0 +. radius *. sin yy
  and dx,dy = border (x0 fby x) (y0 fby y) (dx0 fby dx) (dy0 fby dy)
  in x,y;;


let main_sinusoide center radius speed = 
  let x,y = sinusoide center radius speed in
  let x = truncate x in
  let y = truncate y in
  clear_plot (x fby x) (y fby y);
  plot x y;;



(* dynamic creation *)
let whilenot c =
  let rec w = not c & (true fby w) in
  w;;

let rec balls clicks =
  (* creation of a new ball when click is true *)
  let c = whilenot clicks in
  let x0,y0 = create_random_pos (() when not c) in
  let dx0,dy0 = create_random_speed (() when not c) in
  main_ball (x0,y0,dx0,dy0);
  
  (* recursive calls *)
  let c = false -> not(pre c) in
  merge c (balls (clicks when c)) ();;



let rec circles clicks =
  (* creation of a new circle when click is true *)
  let c = whilenot clicks in
  let pos = create_random_pos (() when not c) in
  let radius = random_float (100.0 when not c) in
  main_circle pos radius;

  (* recursive calls *)
  let c = false -> not(pre c) in
  merge c (circles (clicks when c)) ();;

let rec sinusoides clicks =
  (* creation of a new sinusoide when click is true *)
  let c = whilenot clicks in
  let pos = create_random_pos (() when not c) in
  let speed = create_random_speed (() when not c) in
  let radius = random_float (100.0 when not c) in
  main_sinusoide pos radius speed;

  (* recursive calls *)
  let c = false -> not(pre c) in
  merge c (sinusoides (clicks when c)) ();;


let main mouse_status =
    clear_universe (() when true fby false);
    let clicks = click_on_create_ball mouse_status in
    balls clicks;
    let clicks = click_on_create_circle mouse_status in
    circles clicks;
    let clicks = click_on_create_sinusoide mouse_status in
    sinusoides clicks;;

let main () =
  let mouse_status = make_mouse_status () in
  
  (* quit button *)
  quit (() when click_on_quit mouse_status);
  
  (* playing *)
  reset main mouse_status every (click_on_restart mouse_status);;
