(* les planetes qui tournent et s'attirent *)
(* la force d'attraction entre deux points p1 et p2 *)
(* de masses m1 et m2 et se'pare'es par une distance d *)
(* est g.m1.m2/d^2 *)

open Planetes_data;;

let t = 0.01
let g = 10.0

(* integration *)

let integr init dx = x
    where rec x =  sum (mult t dx)  (init fby x)

(* calcul de la force exerce'e par p2 sur p1 *)
let force p1 p2 =
  let d = distance (pos p1) (pos p2) in
  let force = if d > (radius p2) 
              then g *. (weight (radius p2)) /. (d *. d) 
              else g *. (weight d) /. (d *. d) 
  in mult (force/. d) (dif (pos p2) (pos p1));;

(* calcul de la position courante d'un mobile initial par *)
(* rapport a` p *)
let planete init initp p1 p2 p3 =
  let rec d2p1 = force (init fby p) p1
  and d2p2 = force (init fby p) p2
  and d2p3 = force (init fby p) p3
  and d2 = sum d2p1 (sum  d2p2  d2p3)
  and pp = integr (pos init) (integr initp d2)
  and p = new_planete (init fby p) pp
  in p;;
  
(* l'ensemble des quatres planetes *)
let planetes init1 dp1 init2 dp2 init3 dp3  init4 dp4 =
  let rec p1 = planete init1 dp1 (init2 fby p2) (init3 fby p3) (init4 fby p4)
  and p2 = planete init2 dp2 (init1 fby p1) (init3 fby p3) (init4 fby p4)
  and p3 = planete init3 dp3 (init1 fby p1) (init2 fby p2) (init4 fby p4)
  and p4 = planete init4 dp4 (init1 fby p1) (init2 fby p2) (init3 fby p3) in
  (p1,p2,p3,p4);;

(* la fonction principale *)
let main () =
  let p1 = make_planete red    2.0  (make_point 120.0 120.0 0.0)
  and p2 = make_planete green  3.0  (make_point 250.0 250.0 0.0)
  and p3 = make_planete blue   7.0  (make_point  110.0 110.0 0.0)
  and p4 = make_planete yellow 20.0 (make_point 200.0 200.0  0.0) in
  let p1,p2,p3,p4 = planetes p1 (make_point (-.110.0) 100.0 100.0) 
                             p2 (make_point 100.0  (-.110.0) 0.0)
                             p3 (make_point  (-.110.0) 100.0  0.0)
                             p4 (make_point 10.0 (-.5.0) 0.0)   in
  draw (p1 fby p1) p1;
  draw(p2 fby p2) p2;
  draw (p3 fby p3) p3;
  draw (p4 fby p4) p4;;
