open Graphics;;

type color = Graphics.color

type point = 
    { x : float; y : float ; z : float};;

let make_point x y z = {x = x; y = y; z = z};;

let sum p1 p2 = {x = p1.x +. p2.x; y = p1.y +. p2.y; z = p1.z +. p2.z};;

let dif p1 p2 = {x = p1.x -. p2.x; y = p1.y -. p2.y; z = p1.z -. p2.z};;

let mult k p1 = {x = k*. p1.x ; y = k*. p1.y ; z = k*. p1.z };;

let zero = {x =0.0; y = 0.0; z = 0.0};;

type planete =
    { pos : point;
      radius:float;
      color: color};;

let mv = 10.0
and pi = 3.1416;;

let weight rad = (mv *. (4.0) /. (3.0)) *. pi *. (rad ** 3.0)
and radius p = p.radius
and pos p    = p.pos;;

let make_planete c r p = 
  {pos = p; radius = r; color = c};;

let new_planete p pp = 
  { radius = p.radius;
    color = p.color;
    pos = pp };;

let distance p1 p2 =
  sqrt (((p2.x -. p1.x) ** 2.0) 
	  +. ((p2.y -. p1.y) ** 2.0)
	  +. ((p2.z -. p1.z) ** 2.0))
;;

(* fonctions graphiques *)
let get_cursor () =
  let cx,cy = mouse_pos() in
  float cx,float cy;;

let draw color p = 
  let x = truncate (p.pos.x) in
  let y  = truncate (p.pos.y) in
  set_color color;
  moveto x y;
  fill_circle x y (truncate p.radius);;


let red = red
and green = green
and blue = blue
and yellow = yellow;;

let draw pp p =
  if pp <> p
  then begin
    draw background pp;
    draw p.color p     
  end;;
  
(* initialisation *)
open_graph "";;
