(* graphical library for lucid-synchrone *)

open Xsimul;;

let raising_edge x = x -> not (pre x) & x ;;
let falling_edge x = raising_edge (not x) ;;

let flip_flop x =
  let rec ff = x -> if x then not (pre ff) else pre ff in 
  ff ;;

let button_clic (x,y,sx,sy) (((mx,my),b),(kp,k)) =
  ((mx>x) & (mx<x+sx) & (my>y) & (my<y+sy), b) ;;

(* simple button *)
let simple_button pos label mouse_status = 
  let (mp, mc) = button_clic pos mouse_status  in

  let rmc = raising_edge mc in
  let fmc = falling_edge mc in
  let fmp = falling_edge mp in

  let rec on = ((mp & rmc) or (false -> pre on)) & not ((mp & fmc) or fmp) in
  let out = mp & rmc in

  let cond = (mp & rmc) or (mp & fmc) or fmp in
  display_button pos label on when cond;
  out ;;

(* flip flop button *)
let ff_button pos label_on label_off mp mc res =
  let rmc = raising_edge mc in
  let fmc = falling_edge mc in
  let fmp = falling_edge mp in
  
  let rec on = ((mp & rmc) or (false -> pre on)) 
      & not ((mp & fmc) or fmp) in
  let out = false -> flip_flop (mp & rmc) in

  let label = if out then  label_on else label_off in
  let cond = res or (mp & rmc) or (mp & fmc) or fmp in
  display_button pos label (out or on) when cond;
  out ;;

let flipflop_button  pos label_on label_off mouse_status sreset =
  let (mp, mc) = button_clic pos mouse_status in
  reset 
    ff_button pos label_on label_off mp mc sreset 
  every 
    sreset ;;

(* a graphical entry *)
let entry pos val_def mp kp k res = 
  let cond = mp & kp in
  let rec out =
    merge cond
      (cons_str (val_def -> pre out) k when cond) 
      ((val_def -> pre out) when not cond)
  in
  out ;;

(* integere *)    
let i_entry pos val_def mp kp k res =
  let rec new_value =  
    ios (entry pos (string_of_int val_def) mp kp k res) 
      (val_def -> pre new_value) in
  x_in_int pos new_value;
  new_value ;;
    
let int_entry pos val_def (((mx,my),b),(kp,k)) sreset =
  let (mp, mc) = button_clic pos (((mx,my),b),(kp,k)) in
  reset 
    i_entry pos val_def mp kp k sreset 
  every
    (mc & mp) or sreset ;;    

(* floats *)
let f_entry pos val_def mp kp k res =
  let rec new_value = 
    fos (entry pos (string_of_float val_def) mp kp k res) 
      (val_def -> pre new_value) in
  x_in_float pos new_value;
  new_value ;;
    
let float_entry pos val_def (((mx,my),b),(kp,k)) sreset =
  let (mp, mc) = button_clic pos (((mx,my),b),(kp,k)) in
  reset 
    f_entry pos val_def mp kp k sreset 
  every
    (mc & mp) or sreset ;;  

(* strings *)
let s_entry pos val_def mp kp k res =
  let rec new_value = entry pos val_def mp kp k res in
  x_in_string pos new_value;
  new_value ;;
    
let string_entry pos val_def (((mx,my),b),(kp,k)) sreset =
  let (mp, mc) = button_clic pos (((mx,my),b),(kp,k)) in
  reset 
    s_entry pos val_def mp kp k sreset 
  every
    (mc & mp) or sreset ;;  


