open Misc;;
open Modules;;
open Simul ;;

let x_simulation = ref false ;;
let list_of_input_files = ref ([] : string list) ;;
let simulation_node = ref ([] : string list);;

let set_list_of_input_files input = 
	list_of_input_files := input :: !list_of_input_files;;
let set_simulation_node n = simulation_node := [n];;

let t_doc = "Text-mode simulator (default)"
and x_doc = "X window simulator"
and s_doc = "<node> simulate node <node>"
and errmsg = "Options are :" ;;

let rec load_all_interface = function
   [] -> ()
 | t::q -> 
	if Filename.check_suffix t ".lzi" then (
		open_module (Filename.chop_suffix t ".lzi"); 
	  load_all_interface q 
	) else 
		raise (Arg.Bad ("don't know what to do with "^t^"\n
				Only accept .lzi files."))
;;  

let main () = 
  Arg.parse [
  "-t", Arg.Clear x_simulation, t_doc;
  "-x", Arg.Set x_simulation, x_doc;
  "-s", Arg.String set_simulation_node, s_doc ]
    set_list_of_input_files errmsg ;
    load_all_interface !list_of_input_files ;
    let l = List.map (function x -> Filename.chop_suffix x ".lzi") 
		!list_of_input_files in
 
    try (
      if !x_simulation then
        x_simulate l (List.hd !simulation_node)
      else
	simulate l (List.hd !simulation_node)
    ) with 
        Failure "hd" -> 
	 raise (Arg.Bad "don't know which node to simulate (use -s <node>)")
      | Simulator_error "Bad type" ->
         prerr_string "I can't simulate a node of this type \n";
         prerr_string "Currently, only the primitive types can be simulated\n";
         exit 12
;;

main (); exit 0;
